const os = require('os')
const express = require('express');
const request = require('request');
const { generateToken04 } = require('./utils/getToken04');
const { GenerateUASignature, getServerURL } = require('./utils/gameToken');
const { appConfig } = require('./utils/config');
const app = express();
//端口号
const port = 3020;

const bodyParser = require('body-parser');
const json = express.json({ type: '*/json' });

app.use(json);
app.use(bodyParser.urlencoded({ extended: false }));

app.all('*', function (req, res, next) {
    // 部署服务器时需注释掉
    // 设置响应头，允许跨域请求
    res.header('Access-Control-Allow-Origin', '*')
    // 设置响应头，允许的请求头类型
    res.header('Access-Control-Allow-Headers', 'Authorization,X-API-KEY, Origin, X-Requested-With, Content-Type, Accept, Access-Control-Request-Method' )
    // 设置响应头，允许的请求方法
    res.header('Access-Control-Allow-Methods', 'GET, POST, OPTIONS, PATCH, PUT, DELETE')
    // 设置响应头，允许的请求方法
    res.header('Allow', 'GET, POST, PATCH, OPTIONS, PUT, DELETE')

    if (req.method === 'OPTIONS') {
        // 如果请求方法为OPTIONS，则直接返回200状态码
        res.sendStatus(200);
    } else {
        // 否则，继续执行下一个中间件或路由处理函数
        next();
    }
});

app.get('/api/test', (req, res) => {
    res.send('hello client server');
});

app.post('/api/getUserCurrency', async (req, res) => {
    // 定义一个POST请求的路由，路径为'/api/getUserCurrency'
    const AppId = +req.body.AppId; // 从请求体中获取AppId，并转换为数字类型
    const data = await requestServer(
        {
            UserId: req.body.UserId, // 从请求体中获取UserId
            MiniGameId: req.body.MiniGameId, // 从请求体中获取MiniGameId
            AppId, // 将AppId作为对象的属性值
        },
        getServerURL('DescribeUserCurrency', AppId), // 调用getServerURL函数获取服务器URL，并传入参数'DescribeUserCurrency'和AppId
    );
    res.send(data); // 将获取到的数据作为响应发送给客户端
});

app.post('/api/exchangeUserCurrency', async (req, res) => {
    // 处理POST请求，路径为'/api/exchangeUserCurrency'
    const AppId = +req.body.AppId; // 获取请求体中的AppId，并转换为数字类型
    const OutOrderId = req.body.OutOrderId || `out-order-id:${Date.now()}`;
    const data = await requestServer(
        {
            CurrencyDiff: req.body.CurrencyDiff, // 变化金币，正数增加，负数减少
            UserId: req.body.UserId, // 用户ID
            MiniGameId: req.body.MiniGameId, // 游戏ID
            OutOrderId, // 订单号
        },
        getServerURL('ExchangeUserCurrency', AppId),
    );
    res.send(data);
});

app.post('/api/getOrderInfo', async (req, res) => {
    // 获取订单信息
    const AppId = +req.body.AppId;
    const OrderId = +req.body.OrderId;
    const OutOrderId = req.body.OutOrderId;
    const UserId = req.body.UserId;
    const data = await requestServer(
        {
            UserId,
            OrderId,
            OutOrderId,
        },
        getServerURL('DescribeOrderInfo', AppId),
    );
    res.send(data);
});

app.post('/api/reportGameInfo', async (req, res) => {
    // 此接口供zego服务调用，用于上报对局信息
    const SignatureVersion = req.body.SignatureVersion;
    const AppId = +req.body.AppId;
    const SignatureNonce = req.body.SignatureNonce;
    const Timestamp = +req.body.Timestamp;
    const Signature = req.body.Signature;

    console.log(req.body);
    const checkSignature = GenerateUASignature(AppId, appConfig[AppId].serverSecret, Timestamp, Signature).Signature;
    console.log('checkSignature', checkSignature);
    res.send({ Code: 0, Message: 'success' });
    // if (SignatureVersion === '2.0' && checkSignature === SignatureNonce) {
    //     res.send({ Code: 0, Message: 'success' });
    // } else {
    //     res.send({ Code: 7203001, Message: 'error' });
    // }
});

app.post('/api/token', (req, res) => {
    // 获取用户token信息
    const appID = req.body.app_id || 1739272706;
    const serverSecret = appConfig[appID].serverSecret;
    const effectiveTimeInSeconds = req.body.seconds || 3600;
    const payload = '';
    res.send(generateToken04(appID, req.body.user_id, serverSecret, effectiveTimeInSeconds, payload));
});

app.post('/api/getGameLaunchCode', async (req, res) => {
    // 获取游戏启动码
    const AppId = req.body.AppId;
    console.log(req.body);
    const data = await requestServer(
        {
            RoomId: req.body.RoomId, // 变化金币，正数增加，负数减少
            MiniGameId: req.body.MiniGameId, // 用户ID
            AnchorId: req.body.AnchorId, // 游戏ID
            Nickname: req.body.Nickname, // 订单号
            Avatar: req.body.Avatar,
            Sex: +req.body.Sex,
        },
        getServerURL('DescribeGameLaunchCode', AppId),
    );
    res.send(data);
})

const requestServer = (data, host = 'https://mini-game-api.zego.im') => {
    console.log(data);
    return new Promise((resolve, reject) => {
        request(
            {
                url: host,
                method: 'POST',
                headers: {
                    'content-type': 'application/json',
                },
                body: JSON.stringify(data),
            },
            function (error, response, body) {
                console.log('error', error);
                console.log('response', response);
                console.log('body', body);
                if (!error && response.statusCode == 200) {
                    resolve(body);
                } else {
                    reject(error);
                    throw new Error(error);
                }
            },
        );
    });
};

const getIP = function() {
    let netDict = os.networkInterfaces();
    for (const devName in netDict) {
        let netList = netDict[devName];
        for (let i = 0; i < netList.length; i++) {
            let { address, family, internal } = netList[i];
            if (family === 'IPv4' && address !== '127.0.0.1' && !internal) {
                return address;
            }
        }
    }
}

app.listen(port, () => console.log('server is start,Address is', `http://${getIP()}:${port}`));
